﻿Partial Public Class MainPage
    Inherits PhoneApplicationPage
    ' XNA objects for record and playback
    Private microphone As Microphone
    Private playback As DynamicSoundEffectInstance

    ' Used for storing captured buffers
    Private memoBufferCollection As New List(Of Byte())()

    ' Used for displaying stored memos
    Private memoFiles As New ObservableCollection(Of MemoInfo)()

    ' Data context of record button
    Private spaceTime As New SpaceTime()

    Public Sub New()
        InitializeComponent()

        ' Create new Microphone and set event handler
        microphone = microphone.Default
        AddHandler microphone.BufferReady, AddressOf OnMicrophoneBufferReady

        ' Create new DynamicSoundEffectInstace for playback
        playback = New DynamicSoundEffectInstance(microphone.SampleRate, AudioChannels.Mono)
        AddHandler playback.BufferNeeded, AddressOf OnPlaybackBufferNeeded

        ' Enumerate existing memo waveform files in isolated storage
        Using storage = IsolatedStorageFile.GetUserStoreForApplication()
            ' Show filenames with most recent first
            Dim filenames() = storage.GetFileNames()
            Array.Sort(filenames)
            Array.Reverse(filenames)

            For Each filename In filenames
                Using stream = storage.OpenFile(filename, FileMode.Open, FileAccess.Read)
                    Dim duration = microphone.GetSampleDuration(CInt(stream.Length))
                    Dim memoInfo As New MemoInfo(filename, stream.Length, duration)
                    memoFiles.Add(memoInfo)
                End Using
            Next filename
        End Using

        ' Set memo collection to ListBox
        memosListBox.ItemsSource = memoFiles

        ' Set-up record button
        recordButton.DataContext = spaceTime
        UpdateRecordButton(False)
    End Sub

    Private Sub UpdateRecordButton(ByVal isRecording As Boolean)
        If Not isRecording Then
            Using storage = IsolatedStorageFile.GetUserStoreForApplication()
                spaceTime.Space = storage.AvailableFreeSpace
            End Using
        Else
            spaceTime.Space = memoBufferCollection.Count * microphone.GetSampleSizeInBytes(microphone.BufferDuration)
        End If
        spaceTime.Time = microphone.GetSampleDuration(CInt(Math.Min(spaceTime.Space, Int32.MaxValue)))

        recordButtonContent1.Visibility = If(isRecording, Visibility.Collapsed, Visibility.Visible)
        recordButtonContent2.Visibility = If(isRecording, Visibility.Visible, Visibility.Collapsed)
    End Sub

    Private Sub OnRecordButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        If microphone.State = MicrophoneState.Stopped Then
            ' Clear the collection for storing buffers
            memoBufferCollection.Clear()

            ' Stop any playback in progress (not really necessary, but polite I guess)
            playback.Stop()

            ' Start recording
            microphone.Start()
        Else
            StopRecording()
        End If

        ' Update the record button
        Dim isRecording = microphone.State = MicrophoneState.Started
        UpdateRecordButton(isRecording)
    End Sub

    Private Sub StopRecording()
        ' Get the last partial buffer
        Dim sampleSize = microphone.GetSampleSizeInBytes(microphone.BufferDuration)
        Dim extraBuffer(sampleSize - 1) As Byte
        Dim extraBytes = microphone.GetData(extraBuffer)

        ' Stop recording
        microphone.Stop()

        ' Create MemoInfo object and add at top of collection
        Dim totalSize = memoBufferCollection.Count * sampleSize + extraBytes
        Dim duration = microphone.GetSampleDuration(totalSize)
        Dim memoInfo As New MemoInfo(Date.UtcNow, totalSize, duration)
        memoFiles.Insert(0, memoInfo)

        ' Save data in isolated storage
        Using storage = IsolatedStorageFile.GetUserStoreForApplication()
            Using stream = storage.CreateFile(memoInfo.FileName)
                ' Write buffers from collection
                For Each buffer In memoBufferCollection
                    stream.Write(buffer, 0, buffer.Length)
                Next buffer

                ' Write partial buffer
                stream.Write(extraBuffer, 0, extraBytes)
            End Using
        End Using

        ' Scroll to show new MemoInfo item
        memosListBox.UpdateLayout()
        memosListBox.ScrollIntoView(memoInfo)
    End Sub

    Private Sub OnMicrophoneBufferReady(ByVal sender As Object, ByVal args As EventArgs)
        ' Get buffer from microphone and add to collection
        Dim buffer(microphone.GetSampleSizeInBytes(microphone.BufferDuration) - 1) As Byte
        Dim bytesReturned = microphone.GetData(buffer)
        memoBufferCollection.Add(buffer)

        UpdateRecordButton(True)

        ' Check for 10-minute recording limit.
        ' With the default sample rate of 16000, this is about 19M,
        '      which takes a few seconds to record to isolated storage.
        ' Probably don't want to go much higher without saving the
        '      file incrementally, and providing more protection 
        '      against storage problems.
        If spaceTime.Time > TimeSpan.FromMinutes(10) Then
            StopRecording()
            UpdateRecordButton(False)
        End If
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        ' If navigating from, stop recording
        If microphone.State = MicrophoneState.Started Then StopRecording()

        MyBase.OnNavigatedFrom(args)
    End Sub

    Private Sub OnPlayButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Get clicked item
        Dim btn = TryCast(sender, Button)
        Dim clickedMemoInfo = TryCast(btn.Tag, MemoInfo)
        memosListBox.SelectedItem = clickedMemoInfo

        ' If playing, pause
        If clickedMemoInfo.IsPlaying Then
            playback.Pause()
            clickedMemoInfo.IsPlaying = False
            clickedMemoInfo.IsPaused = True
            ' If paused, resume
        ElseIf clickedMemoInfo.IsPaused Then
            playback.Resume()
            clickedMemoInfo.IsPlaying = True
            clickedMemoInfo.IsPaused = False
            ' Otherwise, start playing
        Else
            ' If another one is playing, stop it
            playback.Stop()

            ' Fetch the waveform data from isolated storage and play it
            Using storage = IsolatedStorageFile.GetUserStoreForApplication()
                Using stream = storage.OpenFile(clickedMemoInfo.FileName,
                                                FileMode.Open,
                                                FileAccess.Read)
                    Dim buffer(CInt(stream.Length - 1)) As Byte
                    stream.Read(buffer, 0, buffer.Length)
                    playback.SubmitBuffer(buffer)
                End Using
            End Using
            playback.Play()

            ' Update the buttons in the ListBox
            For Each memoInfo As MemoInfo In memosListBox.Items
                memoInfo.IsPlaying = memoInfo Is clickedMemoInfo
                memoInfo.IsPaused = False
            Next memoInfo
        End If
    End Sub

    Private Sub OnPlaybackBufferNeeded(ByVal sender As Object, ByVal args As EventArgs)
        ' The whole buffer has been submitted for playing, 
        '  so this merely updates the play button if no buffers are pending
        If playback.PendingBufferCount = 0 Then
            For Each memoInfo As MemoInfo In memosListBox.Items
                memoInfo.IsPlaying = False
                memoInfo.IsPaused = False
            Next memoInfo
        End If
    End Sub

    Private Sub OnMemosListBoxSelectionChanged(ByVal sender As Object,
                                               ByVal args As SelectionChangedEventArgs)
        ' Delete button is enabled only if item is selected
        deleteButton.IsEnabled = memosListBox.SelectedIndex <> -1
    End Sub

    Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Show the confirm buttons
        deleteButtonContent1.Visibility = Visibility.Collapsed
        deleteButtonContent2.Visibility = Visibility.Visible
    End Sub

    Private Sub OnDeleteButtonCancelClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Show the regular delete button
        deleteButtonContent1.Visibility = Visibility.Visible
        deleteButtonContent2.Visibility = Visibility.Collapsed
    End Sub

    Private Sub OnDeleteButtonOkClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Get the selected item
        Dim memoInfo = TryCast(memosListBox.SelectedItem, MemoInfo)
        memosListBox.UpdateLayout()
        memosListBox.ScrollIntoView(memoInfo)

        ' Stop it if it's playing
        If memoInfo.IsPlaying OrElse memoInfo.IsPaused Then playback.Stop()

        ' Delete it from isolated storage
        Using storage = IsolatedStorageFile.GetUserStoreForApplication()
            storage.DeleteFile(memoInfo.FileName)
        End Using

        ' Delete it from the local collection
        memoFiles.Remove(memoInfo)

        ' Cleanup
        OnDeleteButtonCancelClick(sender, args)
    End Sub
End Class